@@ -18,7 +18,7 @@
  * points, and creates a model based forward printer profile
  * (Device -> CIE + spectral), based on Neugenbauer equations.
  * It is designed to handle an arbitrary number of colorants,
- * and in the future, (optionaly) create an aproximate ink overlap/mixing model
+ * and in the future, (optionally) create an aproximate ink overlap/mixing model
  * to allow synthesis of a forward model for a hyperthetical
  * similar printing process with aditional inks.
  *
@@ -262,7 +262,7 @@ make_output_mpp(
 	icg->add_other(icg, "CTI3"); 	/* our special input type is Calibration Target Information 3 */
 
 	if (icg->read_name(icg, inname))
-		error("CGATS file read error : %s",icg->err);
+		error("CGATS file read error : %s",icg->e.m);
 
 	if (icg->ntables == 0 || icg->t[0].tt != tt_other || icg->t[0].oi != 0)
 		error ("Input file isn't a CTI3 format file");
@@ -587,7 +587,7 @@ make_output_mpp(
 
 	/* create and write the cgats profile */
 	if (p->write_mpp(p, outname, dolab))
-		error("Write error : %s",p->err);
+		error("Write error : %s",p->e.m);
 
 	/* Check the forward profile accuracy against the data points */
 	if (verb || verify) {
@@ -762,7 +762,7 @@ make_output_mpp(
 			error("Failed to create an mpp");
 
 		if (p2->read_mpp(p2, outname))
-			error("Read error : %s",p2->err);
+			error("Read error : %s",p2->e.m);
 
 		{
 			/* Set just PCS and use XYZ model */
@@ -921,7 +921,7 @@ make_output_mpp(
 		}
 
 		if (p2->write_mpp(p2, "xxxx.mpp", dolab))
-			error("Write error : %s",p2->err);
+			error("Write error : %s",p2->e.m);
 
 		p2->del(p2);
 	}
