@@ -1,6 +1,6 @@
 
 /* 
- * Argyll Color Correction System
+ * Argyll Color Management System
  * Multi-dimensional regularized splines data fitter
  *
  * Author: Graeme W. Gill
@@ -46,10 +46,15 @@
 
 #ifdef DEBUGLU
 # define DEBLU(xxxx) printf xxxx
+char *icmPiv(int di, int *p);
+char *icmPdv(int di, double *p);
+char *icmPfv(int di, float *p);
 #else
 # define DEBLU(xxxx)
 #endif
 
+#undef USING_INTERP_NL		/* [und] */
+
 /* Implemeted in this file: */
 rspl *new_rspl(int flags, int di, int fdi);
 static void free_rspl(rspl *s);
@@ -64,7 +69,9 @@ static int *get_res(rspl *s);
 static int within_restrictedsize(rspl *s);
 static int interp_rspl_sx(rspl *s, co *pp);
 static int part_interp_rspl_sx(rspl *s, co *p1, co *p2);
+#ifdef USING_INTERP_NL
 static int interp_rspl_nl(rspl *s, co *p);
+#endif
 int is_mono(rspl *s);
 static int set_rspl(rspl *s, int flags, void *cbctx,
                         void (*func)(void *cbctx, double *out, double *in),
@@ -165,9 +172,8 @@ new_rspl(
 	/* Set pointers to methods in this file */
 	s->del           = free_rspl;
 	s->interp        = interp_rspl_sx;	/* Default to simplex interp */
-#ifdef NEVER
-#define USING_INTERP_NL
-printf("!!!! rspl.c using interp_rspl_nl !!!!");
+#ifdef USING_INTERP_NL
+# pragma message("!!!!!!!!! USING_INTERP_NL defined !!!!!!!!!!!")
 	s->interp        = interp_rspl_nl;
 #endif
 	s->part_interp   = part_interp_rspl_sx;
@@ -482,7 +488,7 @@ co *p			/* Input value and returned function value */
 			we[e] = t - (double)mi;		/* 1.0 - weight */
 //if (rspldb && di == 3) printf("~1 e = %d, ix = %d, we = %f\n", e, mi, we[e]);
 		}
-		DEBLU(("ix %d, we %s\n", (gp - s->g.a)/s->g.pss, icmPdv(di, p->p)));
+		DEBLU(("ix %d, we %s\n", (int)(gp - s->g.a)/s->g.pss, icmPdv(di, p->p)));
 	}
 
 	/* Do selection sort on coordinates */
@@ -513,21 +519,21 @@ co *p			/* Input value and returned function value */
 		for (f = 0; f < fdi; f++)
 			p->v[f] = w * gp[f];
 
-		DEBLU(("ix %d: w %f * val %s\n", (gp - s->g.a)/s->g.pss, w, icmPfv(fdi,gp)));
+		DEBLU(("ix %d: w %f * val %s\n", (int)(gp - s->g.a)/s->g.pss, w, icmPfv(fdi,gp)));
 
-		for (e = di-1; e > 0; e--) {		/* Middle verticies */
+		for (e = di-1; e > 0; e--) {		/* Middle vertices */
 			w = we[si[e]] - we[si[e-1]];
 			gp += s->g.fci[si[e]];			/* Move to top of cell in next largest dimension */
 			for (f = 0; f < fdi; f++)
 				p->v[f] += w * gp[f];
-			DEBLU(("ix %d: w %f * val %s\n", (gp - s->g.a)/s->g.pss, w, icmPfv(fdi,gp)));
+			DEBLU(("ix %d: w %f * val %s\n", (int)(gp - s->g.a)/s->g.pss, w, icmPfv(fdi,gp)));
 		}
 
 		w = we[si[0]];
 		gp += s->g.fci[si[0]];		/* Far corner from base of cell */
 		for (f = 0; f < fdi; f++)
 			p->v[f] += w * gp[f];
-		DEBLU(("ix %d: w %f * val %s\n", (gp - s->g.a)/s->g.pss, w, icmPfv(fdi,gp)));
+		DEBLU(("ix %d: w %f * val %s\n", (int)(gp - s->g.a)/s->g.pss, w, icmPfv(fdi,gp)));
 		DEBLU(("Outval  %s\n", icmPdv(fdi, p->v)));
 	}
 	return rv;
@@ -620,7 +626,7 @@ co *p2			/* optional - return partial derivatives for each input channel */
 			p2[di].p[0] = 1.0;
 		}
 		
-		for (e = di-1; e >= 0; e--) {	/* Middle verticies to far vertex from base */
+		for (e = di-1; e >= 0; e--) {	/* Middle vertices to far vertex from base */
 			int ee = si[e];
 			float *lgp = gp;			/* Last gp[] */
 
@@ -1295,7 +1301,7 @@ co *p							/* Target value */
 			we[e] = t - (double)mi;		/* 1.0 - weight */
 //if (rspldb && di == 3) printf("~1 e = %d, ix = %d, we = %f\n", e, mi, we[e]);
 		}
-		DEBLU(("ix %d, we %s\n", (gp - s->g.a)/s->g.pss, icmPdv(di, p->p)));
+		DEBLU(("ix %d, we %s\n", (int)(gp - s->g.a)/s->g.pss, icmPdv(di, p->p)));
 	}
 
 	/* Do selection sort on coordinates */
@@ -1329,15 +1335,15 @@ co *p							/* Target value */
 		for (f = 0; f < fdi; f++)
 			cout[f] = w * gp[f];
 
-		DEBLU(("ix %d: w %f * val %s\n", (gp - s->g.a)/s->g.pss, w, icmPfv(fdi,gp)));
+		DEBLU(("ix %d: w %f * val %s\n", (int)(gp - s->g.a)/s->g.pss, w, icmPfv(fdi,gp)));
 
-		for (e = di-1; e > 0; e--) {		/* Middle verticies */
+		for (e = di-1; e > 0; e--) {		/* Middle vertices */
 			w = we[si[e]] - we[si[e-1]];
 			ww += w * w;					/* Sum of weights squared */
 			gp += s->g.fci[si[e]];			/* Move to top of cell in next largest dimension */
 			for (f = 0; f < fdi; f++)
 				cout[f] += w * gp[f];
-			DEBLU(("ix %d: w %f * val %s\n", (gp - s->g.a)/s->g.pss, w, icmPfv(fdi,gp)));
+			DEBLU(("ix %d: w %f * val %s\n", (int)(gp - s->g.a)/s->g.pss, w, icmPfv(fdi,gp)));
 		}
 
 		w = we[si[0]];
@@ -1345,7 +1351,7 @@ co *p							/* Target value */
 		gp += s->g.fci[si[0]];			/* Far corner from base of cell */
 		for (f = 0; f < fdi; f++)
 			cout[f] += w * gp[f];
-		DEBLU(("ix %d: w %f * val %s\n", (gp - s->g.a)/s->g.pss, w, icmPfv(fdi,gp)));
+		DEBLU(("ix %d: w %f * val %s\n", (int)(gp - s->g.a)/s->g.pss, w, icmPfv(fdi,gp)));
 		DEBLU(("cout  %s\n", icmPdv(fdi, cout)));
 
 		/* We distribute the correction needed in proportion to the */
@@ -1367,7 +1373,7 @@ co *p							/* Target value */
 			}
 		}
 
-		for (e = di-1; e > 0; e--) {	/* Middle verticies */
+		for (e = di-1; e > 0; e--) {	/* Middle vertices */
 			w = we[si[e]] - we[si[e-1]];
 			gp += s->g.fci[si[e]];				/* Move to top of cell in next largest dimension */
 			for (f = 0; f < fdi; f++) {
