@@ -2,6 +2,7 @@
 
 /*
  * Some system dependent convenience functions.
+ * (Some of this should be moved to numlib/numsup  - i.e. threading support ?)
  * Implemented in unixio.c and ntio.c
  */
 
@@ -21,8 +22,14 @@
  */
 
 #if defined (NT)
+# if !defined(WINVER) || WINVER < 0x0501
+#  if defined(WINVER) 
+#   undef WINVER
+#  endif
+#  define WINVER 0x0501
+# endif
 # if !defined(_WIN32_WINNT) || _WIN32_WINNT < 0x0501
-#  if defined _WIN32_WINNT
+#  if defined(_WIN32_WINNT) 
 #   undef _WIN32_WINNT
 #  endif
 #  define _WIN32_WINNT 0x0501
@@ -255,6 +262,15 @@ kkill_nproc_ctx *kkill_nprocess(char **pname, a1log *log);
 
 #include "xdg_bds.h"
 
+/* - - - - - - - - - - - - - - - - - - -- */
+/* Some web functions */
+
+/* Destination should be strlen(s) * 3 + 1 */
+void encodeurl(char *d, char *s);
+
+/* Destination is smaller than src */
+void decodeurl(char *d, char *s);
+
 /* - - - - - - - - - - - - - - - - - - -- */
 /* Some compatibility functions */
 
