@@ -2,7 +2,7 @@
 #ifndef DISPSUP_H
 
 /* 
- * Argyll Color Correction System
+ * Argyll Color Management System
  * Common display patch reading support.
  *
  * Author: Graeme W. Gill
@@ -24,7 +24,9 @@ struct _disp_win_info {
 #ifdef NT
 	int madvrdisp;			/* NZ for MadVR display */
 #endif
+	int dummydisp;			/* NZ for dummy display */
 	disppath *disp;			/* display to calibrate. */
+	double icalmax;			/* Scale inst. cal. test values by this (0.0 .. 1.0) */
 	int out_tvenc;			/* 1 = use RGB Video Level encoding */
 	int fullscreen;			/* NZ if whole screen should be filled with black */
 	int override;			/* Override_redirect on X11 */
@@ -32,7 +34,7 @@ struct _disp_win_info {
 	double vpatsize;		/* Size of dispwin */
 	double ho, vo;			/* Position of dispwin */
 	dispwin *dw;			/* Display window if already open */
-	dispwin *_dw;			/* Privare window if not already open */
+	dispwin *_dw;			/* Private window if not already open */
 }; typedef struct _disp_win_info disp_win_info;
 
 /* A defauult callback that can be provided as an argument to */
@@ -53,6 +55,7 @@ int ditype,			/* Display type selection character(s) */
 int sditype,		/* Spectro ditype, use ditype if -1 */
 int docbid,			/* NZ to only allow cbid ditypes */
 int tele,			/* NZ for tele mode */
+int ambient,		/* NZ for ambient mode */
 int nadaptive,		/* NZ for non-adaptive mode */
 int noinitcal,		/* NZ to disable initial instrument calibration */
 disppath *screen,	/* Screen to calibrate. */
@@ -61,6 +64,8 @@ ccast_id *ccid,		/* non-NULL for ChromeCast */
 #ifdef NT
 int madvrdisp,		/* NZ for MadVR display */
 #endif
+int dummydisp,		/* NZ for dummy display */
+double icalmax,		/* Scale inst. cal. test values by this (0.0 .. 1.0) */
 int out_tvenc,		/* 1 = use RGB Video Level encoding */
 int fullscreen,		/* NZ if whole screen should be filled with black */
 int override,		/* Override_redirect on X11 */
@@ -78,6 +83,7 @@ typedef struct {
 	char *id;			/* Id string */
 
 	inst_meas_type mtype;	/* Measurement type */
+	inst_meas_cond mcond;	/* Reflective measurement condition */
 
 	int    XYZ_v;
 	double XYZ[3];		/* Colorimeter readings */
@@ -107,7 +113,7 @@ struct _disprd {
 						/* 1X = disable color management cLUT (MadVR) */
 	double cal[3][MAX_CAL_ENT];	/* Calibration being worked through (cal[0][0] < 0.0 or NULL if not used) */
 	int ncal;			/* Number of entries used in cal[] */
-	icmLuBase *fake_lu;
+	icmLuSpace *fake_lu;
 	char *mcallout;		/* fake instrument shell callout */
 //	char *scallout;		/* measurement XYZ value callout */
 	int xtern;			/* Use external (user supplied) values rather than instument read */
@@ -122,6 +128,7 @@ struct _disprd {
 	int refrmode;		/* Refresh display mode, -1 if unknow, 0 = if no, 1 if yes */
 	int cbid;			/* The current Calibration Base display mode ID, 0 if unknown */
 	int tele;			/* NZ for tele mode */
+	int ambient;		/* NZ for ambient mode */
 	int nadaptive;		/* NZ for non-adaptive mode */
 	int highres;		/* Use high res mode if available */
 	double refrate;		/* If != 0.0, set display refresh rate calibration */
@@ -197,7 +204,7 @@ struct _disprd {
 	/* 4 = user hit terminate key */
 	/* 5 = system error */
 	/* 8 = no ambient capability */
-	int (*ambient)(struct _disprd *p,
+	int (*meas_ambient)(struct _disprd *p,
 		double *ambient,	/* return ambient in cd/m^2 */
 		int tc				/* If nz, termination key */
 	);
@@ -233,6 +240,7 @@ int ditype,			/* Display type selection character(s) */
 int sditype,		/* Spectro ditype, use ditype if -1 */
 int docbid,			/* NZ to only allow cbid ditypes */
 int tele,			/* NZ for tele mode */
+int ambient,		/* NZ for ambient mode */
 int nadaptive,		/* NZ for non-adaptive mode */
 int noinitcal,		/* No initial instrument calibration */
 int noinitplace,	/* Don't wait for user to place instrument on screen */
@@ -247,6 +255,7 @@ int *nocm,			/* Return nz if no CM cLUT access. native is set to 0X */
 double cal[3][MAX_CAL_ENT],	/* Calibration set (cal = NULL or cal[0][0] < 0.0 if not to be used) */
 int ncal,			/* number of entries use in cal */
 disppath *screen,	/* Screen to calibrate. */
+double icalmax,		/* Scale instrument calibration test values by this (0.0 .. 1.0) */
 int out_tvenc,		/* 1 = use RGB Video Level encoding */
 int fullscreen,		/* NZ if whole screen should be filled with black */
 int override,		/* Override_redirect on X11 */
@@ -255,6 +264,7 @@ ccast_id *ccid,		/* non-NULL for ChromeCast */
 #ifdef NT
 int madvrdisp,		/* NZ for MadVR display */
 #endif
+int dummydisp,		/* NZ for dummy display */
 char *ccallout,		/* Shell callout on set color */
 char *mcallout,		/* Shell callout on measure color (forced fake) */
 //char *scallout,		/* Shell callout on results of measure color */
