@@ -2,7 +2,7 @@
  /* X-Rite ColorMunki related functions */
 
 /* 
- * Argyll Color Correction System
+ * Argyll Color Management System
  *
  * Author: Graeme W. Gill
  * Date:   12/1/2009
@@ -18,7 +18,7 @@
 
 /* 
    If you make use of the instrument driver code here, please note
-   that it is the author(s) of the code who take responsibility
+   that it is the author(s) of the code who are responsibility
    for its operation. Any problems or queries regarding driving
    instruments with the Argyll drivers, should be directed to
    the Argyll's author(s), and not to any other party.
@@ -114,7 +114,7 @@ munki_init_coms(inst *pp, baud_rate br, flow_control fc, double tout) {
 		return munki_interp_code(p, icoms2munki_err(se));
 	}
 
-	a1logd(p->log, 2, "munki_init_coms: init coms has suceeded\n");
+	a1logd(p->log, 2, "munki_init_coms: init coms has succeeded\n");
 
 	p->gotcoms = 1;
 	return inst_ok;
@@ -122,6 +122,8 @@ munki_init_coms(inst *pp, baud_rate br, flow_control fc, double tout) {
 
 static inst_code
 munki_determine_capabilities(munki *p) {
+	munkiimp *m = (munkiimp *)p->m;
+	munki_state *s = m != NULL ? &m->ms[m->mmode] : NULL;
 
 	/* Set the Munki capabilities mask */
 	p->cap = inst_mode_ref_spot
@@ -161,7 +163,9 @@ munki_determine_capabilities(munki *p) {
 		}
 	}
 
-	p->cap3 = inst3_none;
+	if (s != NULL && s->reflective) {
+		p->cap3 = inst3_filter_UVCut;
+	}
 
 	return inst_ok;
 }
@@ -186,7 +190,7 @@ int *conf_ix
 	if (conf_ix == NULL
 	 || *conf_ix < mk_spos_proj
 	 || *conf_ix > mk_spos_amb) {
-		/* Return current configuration measrement modes */
+		/* Return current configuration measurement modes */
 		ev = munki_getstatus(p, &spos, NULL);
 		if (ev != MUNKI_OK)
 			return munki_interp_code(p, ev);
@@ -526,7 +530,7 @@ munki_interp_error(inst *pp, munki_code ec) {
 		case MUNKI_INT_CREATE_EEPROM_STORE:
 			return "Error in creating EEProm store";
 		case MUNKI_INT_NEW_RSPL_FAILED:
-			return "Creating RSPL object faild";
+			return "Creating RSPL object failed";
 		case MUNKI_INT_CAL_SAVE:
 			return "Unable to save calibration to file";
 		case MUNKI_INT_CAL_RESTORE:
@@ -841,6 +845,44 @@ munki_get_set_opt(inst *pp, inst_opt_type m, ...) {
 		return inst_ok;
 	}
 
+	if (m == inst_opt_set_custom_filter) {
+		munkiimp *imp = (munkiimp *)p->m;
+		va_list args;
+		xspect *sp = NULL;
+
+		va_start(args, m);
+
+		sp = va_arg(args, xspect *);
+
+		va_end(args);
+
+		if (sp == NULL || sp->spec_n == 0) {
+			imp->custfilt_en = 0;
+			imp->custfilt.spec_n = 0;
+		} else {
+			imp->custfilt_en = 1;
+			imp->custfilt = *sp;			/* Struct copy */
+		}
+		return inst_ok;
+	}
+
+	if (m == inst_stat_get_custom_filter) {
+		munkiimp *imp = (munkiimp *)p->m;
+		va_list args;
+		xspect *sp = NULL;
+
+		va_start(args, m);
+		sp = va_arg(args, xspect *);
+		va_end(args);
+
+		if (imp->custfilt_en) {
+			*sp = imp->custfilt;			/* Struct copy */
+		} else {
+			sp = NULL;
+		}
+		return inst_ok;
+	}
+
 	if (!p->gotcoms)
 		return inst_no_coms;
 	if (!p->inited)
