@@ -191,7 +191,7 @@ struct _callback {
 	int verb;				/* Verbosity */
 	int total, count, last;	/* Progress count information */
 	rspl *r;				/* correction transform */
-	icmLuBase *rd_luo;		/* Existing abstract profile (NULL if none) */
+	icmLuSpace *rd_luo;		/* Existing abstract profile (NULL if none) */
 	gamut *dev_gam;			/* Gamut of output device (NULL if none) */
 }; typedef struct _callback callback;
 
@@ -201,7 +201,9 @@ struct _callback {
 
 /* New CLUT table */
 /* Correct for PCS errors */
-void PCSp_PCSp(void *cntx, double *out, double *in) {
+void PCSp_PCSp(void *cntx, double *out, double *in
+, int tn
+) {
 	callback *p = (callback *)cntx;
 	co pp;
 
@@ -220,7 +222,7 @@ void PCSp_PCSp(void *cntx, double *out, double *in) {
 #ifdef COMPLOOKUP
 	/* Compound with previous correction */
 	if (p->rd_luo != NULL) {
-		p->rd_luo->lookup(p->rd_luo, out, out);			/* Previous correction */
+		p->rd_luo->lookup_fwd(p->rd_luo, out, out);			/* Previous correction */
 	}
 #endif
 
@@ -278,6 +280,8 @@ main(int argc, char *argv[]) {
 	xspect cust_illum;					/* Custom illumination spectrum */
 	icxObserverType obType = icxOT_CIE_2012_2;
 	xspect custObserver[3];		/* If obType = icxOT_custom */
+	icmTV iccver = ICMTV_DEFAULT;	/* ICC file version to create */
+	icxTransformCreateType icctype = icxTCT_V2V2;	/* ICC profile version to create */
 	callback cb;				/* Callback support stucture for setting abstract profile */
 
 	icmFile *rd_fp = NULL;		/* Existing abstract profile to modify */
@@ -286,6 +290,8 @@ main(int argc, char *argv[]) {
 	icmFile *wr_fp;				/* Modified/created abstract profile to write */
 	icc *wr_icc;
 
+	icmErr err = { 0, { '\000'} };
+
 	int verb = 0;
 	int nogamut = 0;					/* Don't impose a gamut limit */
 	int docreate = 0;					/* Create an initial abstract correction profile */
@@ -305,6 +311,7 @@ main(int argc, char *argv[]) {
 	if (argc < 6)
 		usage("Too few arguments");
 
+
 	/* Process the arguments */
 	for(fa = 1;fa < argc;fa++) {
 		nfa = fa;					/* skip to nfa if next argument is used */
@@ -486,12 +493,14 @@ main(int argc, char *argv[]) {
 				}
 			}
 
+
 			else 
 				usage("Unrecognised flag -%c",argv[fa][1]);
 		} else
 			break;
 	}
 
+
 	/* Grab all the filenames: */
 
 	/* The two CIE value files */
@@ -531,7 +540,7 @@ main(int argc, char *argv[]) {
 		cgf->add_other(cgf, ""); 	/* Allow any signature file */
 	
 		if (cgf->read_name(cgf, cg[n].name))
-			error("CGATS file '%s' read error : %s",cg[n].name,cgf->err);
+			error("CGATS file '%s' read error : %s",cg[n].name,cgf->e.m);
 	
 		if (cgf->ntables < 1)
 			error ("Input file '%s' doesn't contain at least one table",cg[n].name);
@@ -822,6 +831,7 @@ main(int argc, char *argv[]) {
 	/* Possible limiting gamut */
 	if (nogamut == 0) {
 		icmFile *dev_fp;
+		icmErr err = { 0, { '\000'} };
 		icc *dev_icc;
 		xicc *dev_xicc;
 		icxLuBase *dev_luo;
@@ -829,15 +839,15 @@ main(int argc, char *argv[]) {
 
 		/* Open up the device ICC profile, so that we can create a gamut */
 		/* and get an absolute PCS->device conversion */
-		if ((dev_fp = new_icmFileStd_name(dev_name,"r")) == NULL)
-			error ("Can't open file '%s'",dev_name);
+		if ((dev_fp = new_icmFileStd_name(&err,dev_name,"r")) == NULL)
+			error ("Can't open file '%s' (0x%x, '%s')",dev_name,err.c,err.m);
 	
-		if ((dev_icc = new_icc()) == NULL)
-			error("Creation of ICC object failed");
+		if ((dev_icc = new_icc(&err)) == NULL)
+			error("Creation of ICC object failed (0x%x, '%s')",err.c,err.m);
 	
 		/* Read header etc. */
 		if ((rv = dev_icc->read(dev_icc,dev_fp,0)) != 0)
-			error("Reading profile '%s' failed: %d, %s",dev_name,rv,dev_icc->err);
+			error("Reading profile '%s' failed: %d, %s",dev_name,rv,dev_icc->e.m);
 	
 		/* Check that the profile is appropriate */
 		if (dev_icc->header->deviceClass != icSigInputClass
@@ -863,11 +873,11 @@ main(int argc, char *argv[]) {
 		if ((dev_luo = dev_xicc->get_luobj(dev_xicc, ICX_CLIP_NEAREST, icmFwd,
 		     dorel ? icRelativeColorimetric : icAbsoluteColorimetric,
 		     icSigLabData, icmLuOrdNorm, NULL, &ink)) == NULL)
-			error ("%d, %s",dev_xicc->errc, dev_xicc->err);
+			error ("%d, %s",dev_xicc->e.c, dev_xicc->e.m);
 	
 		/* Creat a gamut surface */
 		if ((cb.dev_gam = dev_luo->get_gamut(dev_luo, GAMRES)) == NULL)
-			error ("%d, %s",dev_xicc->errc, dev_xicc->err);
+			error ("%d, %s",dev_xicc->e.c, dev_xicc->e.m);
 
 		dev_luo->del(dev_luo);
 		dev_xicc->del(dev_xicc);
@@ -880,23 +890,23 @@ main(int argc, char *argv[]) {
 	/* ======================= */
 	/* Open up the existing abstract profile that is to be refined. */
 	if (docreate == 0) {
-		if ((rd_fp = new_icmFileStd_name(rd_name,"r")) == NULL)
-			error ("Can't open file '%s'",rd_name);
+		if ((rd_fp = new_icmFileStd_name(&err,rd_name,"r")) == NULL)
+			error ("Can't open file '%s' (0x%x, '%s')",rd_name,err.c,err.m);
 	
-		if ((rd_icc = new_icc()) == NULL)
-			error ("Creation of ICC object failed");
+		if ((rd_icc = new_icc(&err)) == NULL)
+			error ("Creation of ICC object failed (0x%x, '%s')",err.c,err.m);
 	
 		/* Read header etc. */
 		if ((rv = rd_icc->read(rd_icc,rd_fp,0)) != 0)
-			error ("%d, %s",rv,rd_icc->err);
+			error ("%d, %s",rv,rd_icc->e.m);
 	
 		if (rd_icc->header->deviceClass != icSigAbstractClass)
 			error("Input Profile '%s' isn't abstract type",rd_name);
 
-		if ((cb.rd_luo = rd_icc->get_luobj(rd_icc, icmFwd,
+		if ((cb.rd_luo = (icmLuSpace *)rd_icc->get_luobj(rd_icc, icmFwd,
 		        dorel ? icRelativeColorimetric : icAbsoluteColorimetric,
 		        icSigLabData, icmLuOrdNorm)) == NULL)
-				error ("%d, %s",rd_icc->errc, rd_icc->err);
+				error ("%d, %s",rd_icc->e.c, rd_icc->e.m);
 	} else {
 		cb.rd_luo = NULL;
 	}
@@ -944,7 +954,7 @@ main(int argc, char *argv[]) {
 			/* Lookup the current correction applied to the target */
 			if (cb.rd_luo != NULL) {		/* Subsequent pass */
 				double corval[3];
-				cb.rd_luo->lookup(cb.rd_luo, corval, cg[0].pat[i].v);
+				cb.rd_luo->lookup_fwd(cb.rd_luo, corval, cg[0].pat[i].v);
 				icmSub3(ccor, corval, cg[0].pat[i].v);
 				cmag = icmNorm3(ccor);
 #ifdef DEBUG1
@@ -1151,11 +1161,14 @@ main(int argc, char *argv[]) {
 
 	/* ======================= */
 	/* Create new abstract ICC profile */
-	if ((wr_fp = new_icmFileStd_name(wr_name,"w")) == NULL)
-		error ("Can't open file '%s' for writing",wr_name);
+	if ((wr_fp = new_icmFileStd_name(&err,wr_name,"w")) == NULL)
+		error ("Can't open file '%s' for writing (0x%x, '%s')",wr_name,err.c,err.m);
 
-	if ((wr_icc = new_icc()) == NULL)
-		error ("Creation of write ICC object failed");
+	if ((wr_icc = new_icc(&err)) == NULL)
+		error ("Creation of write ICC object failed (0x%x, '%s')",err.c,err.m);
+
+	if (wr_icc->set_version(wr_icc, iccver) != 0)
+		error("set_version %d failed: %d, %s",iccver,wr_icc->e.c,wr_icc->e.m);
 
 	/* Add all the tags required */
 
@@ -1174,27 +1187,27 @@ main(int argc, char *argv[]) {
 	}
 	/* Profile Description Tag: */
 	{
-		icmTextDescription *wo;
+		icmCommonTextDescription *wo;
 		char *dst = "Argyll refine output";
-		if ((wo = (icmTextDescription *)wr_icc->add_tag(
-		           wr_icc, icSigProfileDescriptionTag,	icSigTextDescriptionType)) == NULL) 
-			error("add_tag failed: %d, %s",wr_icc->errc,wr_icc->err);
+		if ((wo = (icmCommonTextDescription *)wr_icc->add_tag(
+		           wr_icc, icSigProfileDescriptionTag,	icmSigCommonTextDescriptionType)) == NULL) 
+			error("add_tag failed: %d, %s",wr_icc->e.c,wr_icc->e.m);
 
-		wo->size = strlen(dst)+1; 	/* Allocated and used size of desc, inc null */
-		wo->allocate((icmBase *)wo);/* Allocate space */
+		wo->count = strlen(dst)+1; 	/* Allocated and used size of desc, inc null */
+		wo->allocate(wo);/* Allocate space */
 		strcpy(wo->desc, dst);		/* Copy the string in */
 	}
 	/* Copyright Tag: */
 	{
-		icmText *wo;
+		icmCommonTextDescription *wo;
 		char *crt = "Copyright the user who created it";
-		if ((wo = (icmText *)wr_icc->add_tag(
-		           wr_icc, icSigCopyrightTag,	icSigTextType)) == NULL) 
-			error("add_tag failed: %d, %s",wr_icc->errc,wr_icc->err);
+		if ((wo = (icmCommonTextDescription *)wr_icc->add_tag(
+		           wr_icc, icSigCopyrightTag,	icmSigCommonTextDescriptionType)) == NULL) 
+			error("add_tag failed: %d, %s",wr_icc->e.c,wr_icc->e.m);
 
-		wo->size = strlen(crt)+1; 	/* Allocated and used size of text, inc null */
-		wo->allocate((icmBase *)wo);/* Allocate space */
-		strcpy(wo->data, crt);		/* Copy the text in */
+		wo->count = strlen(crt)+1; 	/* Allocated and used size of text, inc null */
+		wo->allocate(wo);/* Allocate space */
+		strcpy(wo->desc, crt);		/* Copy the text in */
 	}
 	/* White Point Tag: */
 	{
@@ -1202,57 +1215,67 @@ main(int argc, char *argv[]) {
 		/* Note that tag types icSigXYZType and icSigXYZArrayType are identical */
 		if ((wo = (icmXYZArray *)wr_icc->add_tag(
 		           wr_icc, icSigMediaWhitePointTag, icSigXYZArrayType)) == NULL) 
-			error("add_tag failed: %d, %s",wr_icc->errc,wr_icc->err);
+			error("add_tag failed: %d, %s",wr_icc->e.c,wr_icc->e.m);
 
-		wo->size = 1;
-		wo->allocate((icmBase *)wo);	/* Allocate space */
+		wo->count = 1;
+		wo->allocate(wo);	/* Allocate space */
 		wo->data[0] = icmD50;			/* So absolute/relative rendering is the same */
 	}
 	/* 16 bit pcs -> pcs lut: */
 	{
-		icmLut *wo;
 		int flags = ICM_CLUT_SET_EXACT;	/* Assume we're setting from RSPL's */
 
-		/* Intent 0 = default/perceptual */
-		if ((wo = (icmLut *)wr_icc->add_tag(
-		           wr_icc, icSigAToB0Tag,	icSigLut16Type)) == NULL) 
-			error("add_tag failed: %d, %s",wr_icc->errc,wr_icc->err);
+		int nsigs = 1;
+		icmXformSigs sigs[2];
+		unsigned int inputEnt, outputEnt;
+		unsigned int clutPoints[MAX_CHAN];
 
-		wo->inputChan = 3;
-		wo->outputChan = 3;
-    	wo->clutPoints = clutres;
-    	wo->inputEnt = 256;				/* Not actually used */
-    	wo->outputEnt = 256;
-		wo->allocate((icmBase *)wo);/* Allocate space */
-
-		/* The matrix is only applicable to XYZ input space, */
-		/* so it is not used here. */
-
-		/* Use helper function to do the hard work. */
-		if (cb.verb) {
-			int extra;
-			for (cb.total = 1, i = 0; i < 3; i++, cb.total *= wo->clutPoints)
-				; 
-			/* Add in cell center points */
-			for (extra = 1, i = 0; i < wo->inputChan; i++, extra *= (wo->clutPoints-1))
-				;
-			cb.total += extra;
-			cb.count = 0;
-			cb.last = -1;
-			printf(" 0%%"), fflush(stdout);
+		switch (icctype) {
+			default:
+				sigs[0].sig = icSigAToB0Tag;
+				sigs[0].ttype = icSigLut16Type;
+				/* Default ICC Version used */
+				break;
 		}
 
 #ifdef COMPLOOKUP
 		/* Compound with previous correction */
 		if (cb.rd_luo != NULL)
-			flags = ICM_CLUT_SET_APXLS;	/* Won't be least squares, so do extra sampling */
+			flags |= ICM_CLUT_SET_APXLS;	/* Won't be least squares, so do extra sampling */
 #endif
 
-		if (wo->set_tables(wo,
+    	inputEnt = 256;
+		for (i = 0; i < 3; i++)
+	    	clutPoints[i] = clutres;
+    	outputEnt = 256;
+
+		/* Use helper function to do the hard work. */
+		if (cb.verb) {
+			int extra;
+			for (cb.total = 1, i = 0; i < 3; cb.total *= clutPoints[i], i++)
+				; 
+			if (flags & ICM_CLUT_SET_APXLS) {
+				/* Add in cell center points */
+				for (extra = 1, i = 0; i < 3; extra *= (clutPoints[i]-1), i++)
+					;
+				cb.total += extra;
+			}
+			cb.count = 0;
+			cb.last = -1;
+			printf(" 0%%"), fflush(stdout);
+		}
+
+		if (wr_icc->create_lut_xforms(
+				wr_icc,
 				flags,
-				&cb,
+				&cb,					/* Callback context */
+				nsigs,					/* Number of tables */
+				sigs,					/* signatures and tag types for each table */
+				2,						/* Bytes per value of AToB or BToA CLUT, 1 or 2 */
+				inputEnt, clutPoints, outputEnt,	/* Table resolutions */
 				icSigLabData, 			/* Input color space */
 				icSigLabData, 			/* Output color space */
+				NULL, NULL,				/* Use default input range */
 				NULL,					/* Linear input transform Lab->Lab' (NULL = default) */
 				NULL, NULL,				/* Use default Maximum range of Lab' values */
 				PCSp_PCSp,				/* Lab' -> Lab' transfer function */
@@ -1260,7 +1283,7 @@ main(int argc, char *argv[]) {
 				NULL,					/* Linear output transform Lab'->Lab */
 				NULL, NULL				/* Default SET_APXLS range */ 
 		) != 0)
-			error("Setting 16 bit Lab->Lab Lut failed: %d, %s",wr_icc->errc,wr_icc->err);
+			error("Setting 16 bit Lab->Lab Lut failed: %d, %s",wr_icc->e.c,wr_icc->e.m);
 
 		if (verb)
 			printf("\n");
@@ -1273,7 +1296,7 @@ main(int argc, char *argv[]) {
 	}
 	/* Write the file out */
 	if ((rv = wr_icc->write(wr_icc,wr_fp,0)) != 0)
-		error ("Write file: %d, %s",rv,wr_icc->err);
+		error ("Write file: %d, %s",rv,wr_icc->e.m);
 	
 	/* ======================================= */
 	
