@@ -24,7 +24,7 @@
 #endif
 
 /*
- * This object provides storage and application of emisive spectral
+ * This object provides storage and application of emissive spectral
  * samples that can be used to compute calibration for suitable
  * colorimeters (such as the i1d3) tuned for particular types of displays.
  * 
@@ -81,8 +81,8 @@ struct _ccss {
 	int no_samp;		/* Number of samples */
 	
 	/* Houskeeping - should switch this to a1log ? */
-	int errc;				/* Error code */
-	char err[200];			/* Error message */
+	icmErr e;				/* Error code & message */
+
 }; typedef struct _ccss ccss;
 
 /* Create a new, uninitialised ccss */
